import os 
import re
import sys
import pathlib
import datetime
import locale


def xstr(string):
    if string is None:
        return ''
    else:
        return str(string)


def str_to_list(s):
    s = s.strip()
    result = []
    for x in s.split(','):
        result.append(x.strip())
    
    return result

def person_sort_name(name, first_delimiter=' '):

    n = name.split()
    if len(n) > 0:
        n.insert(0, n.pop())
        if len(n) == 1:
            return n[0]
        elif len(n) > 1:
            return n[0] + first_delimiter + ' '.join(n[1:])
        return ''
    else:
        return ''


def replace_keywords(s, m):

    def expand_keyword(s, key, val):
        if s.count(key) > 0:
            return s.replace(key, val), True if val else False
        return s, False

    def expand_all(s, m):

        expanded = False

        sorted_m = {}
        for k in sorted(m, key=len, reverse=True):
            sorted_m[k] = m[k]

        for key in sorted_m:
            s, ok = expand_keyword(s, key, sorted_m[key])
            expanded = expanded or ok
        
        if not expanded:
            return ''

        return s

    b_open = -1
    b_close = -1

    # Hack - I do not want to write real parser
    # s = s.replace(r'\{', chr(1)).replace(r'\}', chr(2))

    for i, sym in enumerate(s):
        if sym == '{':
            b_open = i
        elif sym == '}':
            b_close = i
            break
    if b_open >= 0 and b_close > 0 and b_open < b_close:
        s = replace_keywords(s[0:b_open] + expand_all(s[b_open + 1:b_close], m) + s[b_close + 1:], m)
    else:
        s = expand_all(s, m)

    return s


def split_ext(path):
    for ext in ['.fb2.zip']:
        if path.lower().endswith(ext):
            return ext
    return os.path.splitext(path)[1]

def replace_bad_symbols_(path):
    bad_symbols = '<>:"|*?'
    for s in bad_symbols:
        path = path.replace(s, '')
    return path

def normalize_path(path):
    def remove_illegal(s):
        illegal = '<>?:"*'
        for c in illegal:
            s = s.replace(c,'')
        return s

    def normalize_part(s: str) -> str:
        s = s.strip()
        if s.endswith('.'):
            s = s[0:-1]
    
        return remove_illegal(s)

    path = re.sub(r'\.+', '.', path)
    path = re.sub(r'\\+', r'\\', path)
    path = re.sub(r'\/+', '/', path)
    path = path.replace(r'\\', r'\\\\')

    p = pathlib.Path(path)
    clean_p = []

    if p.is_absolute():
        start = 1
        clean_p.append(p.anchor)
    else:
        start = 0
    for part in p.parts[start:]:
        clean_p.append(normalize_part(part))
    
    return str(pathlib.Path(*clean_p))


def get_file_creation_time(file):
    locale.setlocale(locale.LC_ALL, '')

    time = None
    if sys.platform == 'win32':
        time = os.path.getctime(file)
    else:
        time = os.stat(file).st_birthtime

    return datetime.datetime.fromtimestamp(time).strftime('%x %X')

def get_file_modified_time(file):
    locale.setlocale(locale.LC_ALL, '')
    time = os.stat(file).st_mtime
    return datetime.datetime.fromtimestamp(time).strftime('%x %X')



fb2genres = '''<fbgenrestransfer xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:noNamespaceSchemaLocation="http://alexs.ru/fb2/genrestable/GT.xsd">
  <genre value="sf">
    <root-descr lang="en" genre-title="SF, Fantasy" detailed="Alternate History, Cyberpunk, Fantasy, Science Fiction"/>
    <root-descr lang="ru" genre-title="Фантастика, Фэнтэзи" detailed="НФ, Фэнтези, Мистика, Киберпанк"/>
    <subgenres>
      <subgenre value="sf_history">
        <genre-descr lang="en" title="Alternative history"/>
        <genre-descr lang="ru" title="Альтернативная история"/>
        <genre-alt value="fantasy_alt_hist" format="fb2.0"/>
      </subgenre>
      <subgenre value="sf_action">
        <genre-descr lang="en" title="Action SF"/>
        <genre-descr lang="ru" title="Боевая Фантастика"/>
      </subgenre>
      <subgenre value="sf_epic">
        <genre-descr lang="en" title="Epic SF"/>
        <genre-descr lang="ru" title="Эпическая Фантастика"/>
      </subgenre>
      <subgenre value="sf_heroic">
        <genre-descr lang="en" title="Heroic SF"/>
        <genre-descr lang="ru" title="Героическая фантастика"/>
      </subgenre>
      <subgenre value="sf_detective">
        <genre-descr lang="en" title="Detective SF"/>
        <genre-descr lang="ru" title="Детективная Фантастика"/>
      </subgenre>
      <subgenre value="sf_cyberpunk">
        <genre-descr lang="en" title="Cyberpunk"/>
        <genre-descr lang="ru" title="Киберпанк"/>
        <genre-alt value="sf_cyber_punk" format="fb2.0"/>
      </subgenre>
      <subgenre value="sf_space">
        <genre-descr lang="en" title="Space SF"/>
        <genre-descr lang="ru" title="Космическая Фантастика"/>
      </subgenre>
      <subgenre value="sf_social">
        <genre-descr lang="en" title="Social SF"/>
        <genre-descr lang="ru" title="Социальная фантастика"/>
      </subgenre>
      <subgenre value="sf_horror">
        <genre-descr lang="en" title="Horror and Mystic"/>
        <genre-descr lang="ru" title="Ужасы и Мистика"/>
        <genre-alt value="gay_mystery" format="fb2.0"/>
        <genre-alt value="horror" format="fb2.0"/>
        <genre-alt value="horror_antology" format="fb2.0"/>
        <genre-alt value="horror_british" format="fb2.0"/>
        <genre-alt value="horror_fantasy" format="fb2.0"/>
        <genre-alt value="horror_erotic" format="fb2.0"/>
        <genre-alt value="horror_ghosts" format="fb2.0"/>
        <genre-alt value="horror_graphic" format="fb2.0"/>
        <genre-alt value="horror_occult" format="fb2.0"/>
        <genre-alt value="horror_ref" format="fb2.0"/>
        <genre-alt value="horror_usa" format="fb2.0"/>
        <genre-alt value="horror_vampires" format="fb2.0"/>
        <genre-alt value="teens_horror" format="fb2.0"/>
      </subgenre>
      <subgenre value="sf_humor">
        <genre-descr lang="en" title="Humor SF"/>
        <genre-descr lang="ru" title="Юмористическая фантастика"/>
      </subgenre>
      <subgenre value="sf_fantasy">
        <genre-descr lang="en" title="Fantasy"/>
        <genre-descr lang="ru" title="Фэнтези"/>
        <genre-alt value="romance_fantasy" format="fb2.0"/>
        <genre-alt value="romance_sf" format="fb2.0"/>
        <genre-alt value="romance_time_travel" format="fb2.0"/>
      </subgenre>
      <subgenre value="sf">
        <genre-descr lang="en" title="Science Fiction"/>
        <genre-descr lang="ru" title="Научная Фантастика"/>
        <genre-alt value="gaming" format="fb2.0"/>
        <genre-alt value="sf_writing" format="fb2.0"/>
      </subgenre>
      <subgenre value="child_sf">
        <genre-descr lang="en" title="Science Fiction for Kids"/>
        <genre-descr lang="ru" title="Детская Фантастика"/>
        <genre-alt value="teens_sf" format="fb2.0"/>
      </subgenre>
    </subgenres>
  </genre>
  <genre value="detective">
    <root-descr lang="en" genre-title="Detectives, Thrillers" detailed="Police Stories, Ironical, Espionage, Crime"/>
    <root-descr lang="ru" genre-title="Детективы, Боевики" detailed="Полицейские, иронические, шпионские, криминальные"/>
    <subgenres>
      <subgenre value="det_classic">
        <genre-descr lang="en" title="Classical Detective"/>
        <genre-descr lang="ru" title="Классический Детектив"/>
      </subgenre>
      <subgenre value="det_police">
        <genre-descr lang="en" title="Police Stories"/>
        <genre-descr lang="ru" title="Полицейский Детектив"/>
        <genre-alt value="thriller_police" format="fb2.0"/>
      </subgenre>
      <subgenre value="det_action">
        <genre-descr lang="en" title="Action"/>
        <genre-descr lang="ru" title="Боевики"/>
      </subgenre>
      <subgenre value="det_irony">
        <genre-descr lang="en" title="Ironical Detective"/>
        <genre-descr lang="ru" title="Иронический Детектив"/>
      </subgenre>
      <subgenre value="det_history">
        <genre-descr lang="en" title="Historical Detective"/>
        <genre-descr lang="ru" title="Исторический Детектив"/>
      </subgenre>
      <subgenre value="det_espionage">
        <genre-descr lang="en" title="Espionage Detective"/>
        <genre-descr lang="ru" title="Шпионский Детектив"/>
      </subgenre>
      <subgenre value="det_crime">
        <genre-descr lang="en" title="Crime Detective"/>
        <genre-descr lang="ru" title="Криминальный Детектив"/>
      </subgenre>
      <subgenre value="det_political">
        <genre-descr lang="en" title="Political Detective"/>
        <genre-descr lang="ru" title="Политический Детектив"/>
      </subgenre>
      <subgenre value="det_maniac">
        <genre-descr lang="en" title="Maniacs"/>
        <genre-descr lang="ru" title="Маньяки"/>
      </subgenre>
      <subgenre value="det_hard">
        <genre-descr lang="en" title="Hard-boiled Detective"/>
        <genre-descr lang="ru" title="Крутой Детектив"/>
      </subgenre>
      <subgenre value="thriller">
        <genre-descr lang="en" title="Thrillers"/>
        <genre-descr lang="ru" title="Триллеры"/>
        <genre-alt value="thriller_mystery" format="fb2.0"/>
      </subgenre>
      <subgenre value="detective">
        <genre-descr lang="en" title="Detective"/>
        <genre-descr lang="ru" title="Детектив"/>
        <genre-alt value="mystery" format="fb2.0"/>
      </subgenre>
      <subgenre value="sf_detective">
        <genre-descr lang="en" title="Detective SF"/>
        <genre-descr lang="ru" title="Детективная Фантастика"/>
        <genre-alt value="teens_mysteries" format="fb2.0"/>
      </subgenre>
      <subgenre value="child_det">
        <genre-descr lang="en" title="Detectives and Thrillers"/>
        <genre-descr lang="ru" title="Детские Остросюжетные"/>
      </subgenre>
      <subgenre value="love_detective">
        <genre-descr lang="en" title="Detective Romance"/>
        <genre-descr lang="ru" title="Остросюжетные Любовные Романы"/>
      </subgenre>
    </subgenres>
  </genre>
  <genre value="prose">
    <root-descr lang="en" genre-title="Prose" detailed="Classical, History, Contemporary"/>
    <root-descr lang="ru" genre-title="Проза" detailed="Классика, историческая, современная"/>
    <subgenres>
      <subgenre value="prose_classic">
        <genre-descr lang="en" title="Classics Prose"/>
        <genre-descr lang="ru" title="Классическая Проза"/>
        <genre-alt value="literature" format="fb2.0"/>
        <genre-alt value="literature_books" format="fb2.0"/>
        <genre-alt value="literature_british" format="fb2.0"/>
        <genre-alt value="literature_classics" format="fb2.0"/>
        <genre-alt value="literature_drama" format="fb2.0"/>
        <genre-alt value="literature_essay" format="fb2.0"/>
        <genre-alt value="literature_antology" format="fb2.0"/>
        <genre-alt value="literature_saga" format="fb2.0"/>
        <genre-alt value="literature_short" format="fb2.0"/>
        <genre-alt value="literature_usa" format="fb2.0"/>
        <genre-alt value="literature_world" format="fb2.0"/>
      </subgenre>
      <subgenre value="prose_history">
        <genre-descr lang="en" title="Historical Prose"/>
        <genre-descr lang="ru" title="Историческая Проза"/>
        <genre-alt value="literature_history" format="fb2.0"/>
        <genre-alt value="literature_critic" format="fb2.0"/>
      </subgenre>
      <subgenre value="prose_contemporary">
        <genre-descr lang="en" title="Contemporary Prose"/>
        <genre-descr lang="ru" title="Современная Проза"/>
        <genre-alt value="literature_political" format="fb2.0"/>
        <genre-alt value="literature_war" format="fb2.0"/>
        <genre-alt value="ref_writing" format="fb2.0"/>
      </subgenre>
      <subgenre value="prose_counter">
        <genre-descr lang="en" title="Counterculture"/>
        <genre-descr lang="ru" title="Контркультура"/>
        <genre-alt value="literature_gay" format="fb2.0"/>
      </subgenre>
      <subgenre value="prose_rus_classsic">
        <genre-descr lang="en" title="Russian Classics"/>
        <genre-descr lang="ru" title="Русская Классика"/>
        <genre-alt value="literature_rus_classsic" format="fb2.0"/>
        <genre-alt value="prose_rus_classic" format="fb2.0"/>
      </subgenre>
      <subgenre value="prose_su_classics">
        <genre-descr lang="en" title="Soviet Classics"/>
        <genre-descr lang="ru" title="Советская Классика"/>
        <genre-alt value="literature_su_classics" format="fb2.0"/>
      </subgenre>
      <subgenre value="humor_prose">
        <genre-descr lang="en" title="Humor Prose"/>
        <genre-descr lang="ru" title="Юмористическая Проза"/>
      </subgenre>
      <subgenre value="child_prose">
        <genre-descr lang="en" title="Prose"/>
        <genre-descr lang="ru" title="Детская Проза"/>
        <genre-alt value="teens_literature" format="fb2.0"/>
      </subgenre>
    </subgenres>
  </genre>
  <genre value="love">
    <root-descr lang="en" genre-title="Romance" detailed="Historical, Contemporary, Detective"/>
    <root-descr lang="ru" genre-title="Любовные романы" detailed="Исторические, современные, остросюжетные"/>
    <subgenres>
      <subgenre value="love_contemporary">
        <genre-descr lang="en" title="Contemporary Romance"/>
        <genre-descr lang="ru" title="Современные Любовные Романы"/>
        <genre-alt value="romance" format="fb2.0"/>
        <genre-alt value="romance_multicultural" format="fb2.0"/>
        <genre-alt value="romance_series" format="fb2.0"/>
        <genre-alt value="romance_anthologies" format="fb2.0"/>
        <genre-alt value="romance_contemporary" format="fb2.0"/>
        <genre-alt value="literature_women" format="fb2.0"/>
        <genre-alt value="romance_romantic_suspense" format="fb2.0"/>
      </subgenre>
      <subgenre value="love_history">
        <genre-descr lang="en" title="Historical Romance"/>
        <genre-descr lang="ru" title="Исторические Любовные Романы"/>
        <genre-alt value="romance_regency" format="fb2.0"/>
        <genre-alt value="romance_historical" format="fb2.0"/>
      </subgenre>
      <subgenre value="love_detective">
        <genre-descr lang="en" title="Detective Romance"/>
        <genre-descr lang="ru" title="Остросюжетные Любовные Романы"/>
      </subgenre>
      <subgenre value="love_short">
        <genre-descr lang="en" title="Short Romance"/>
        <genre-descr lang="ru" title="Короткие Любовные Романы"/>
      </subgenre>
      <subgenre value="love_erotica">
        <genre-descr lang="en" title="Erotica"/>
        <genre-descr lang="ru" title="Эротика"/>
        <genre-alt value="literature_erotica" format="fb2.0"/>
      </subgenre>
    </subgenres>
  </genre>
  <genre value="adventure">
    <root-descr lang="en" genre-title="Adventure" detailed="Western, Historical, Sea"/>
    <root-descr lang="ru" genre-title="Приключения" detailed="Вестерны, исторические, морские"/>
    <subgenres>
      <subgenre value="adv_western">
        <genre-descr lang="en" title="Western"/>
        <genre-descr lang="ru" title="Вестерны"/>
        <genre-alt value="literature_western" format="fb2.0"/>
      </subgenre>
      <subgenre value="adv_history">
        <genre-descr lang="en" title="History"/>
        <genre-descr lang="ru" title="Исторические Приключения"/>
      </subgenre>
      <subgenre value="adv_indian">
        <genre-descr lang="en" title="Indians"/>
        <genre-descr lang="ru" title="Приключения: Индейцы"/>
      </subgenre>
      <subgenre value="adv_maritime">
        <genre-descr lang="en" title="Maritime Fiction"/>
        <genre-descr lang="ru" title="Морские Приключения"/>
        <genre-alt value="literature_sea" format="fb2.0"/>
      </subgenre>
      <subgenre value="adv_geo">
        <genre-descr lang="en" title="Travel and Geography"/>
        <genre-descr lang="ru" title="Путешествия и География"/>
        <genre-alt value="gay_travel" format="fb2.0"/>
        <genre-alt value="outdoors_travel" format="fb2.0"/>
        <genre-alt value="travel" format="fb2.0"/>
        <genre-alt value="travel_africa" format="fb2.0"/>
        <genre-alt value="travel_asia" format="fb2.0"/>
        <genre-alt value="travel_australia" format="fb2.0"/>
        <genre-alt value="travel_canada" format="fb2.0"/>
        <genre-alt value="travel_caribbean" format="fb2.0"/>
        <genre-alt value="travel_europe" format="fb2.0"/>
        <genre-alt value="travel_guidebook_series" format="fb2.0"/>
        <genre-alt value="travel_lat_am" format="fb2.0"/>
        <genre-alt value="travel_middle_east" format="fb2.0"/>
        <genre-alt value="travel_polar" format="fb2.0"/>
        <genre-alt value="travel_spec" format="fb2.0"/>
        <genre-alt value="travel_usa" format="fb2.0"/>
        <genre-alt value="travel_rus" format="fb2.0"/>
        <genre-alt value="travel_ex_ussr" format="fb2.0"/>
      </subgenre>
      <subgenre value="adv_animal">
        <genre-descr lang="en" title="Nature and Animals"/>
        <genre-descr lang="ru" title="Природа и Животные"/>
        <genre-alt value="child_animals" format="fb2.0"/>
      </subgenre>
      <subgenre value="adventure">
        <genre-descr lang="en" title="Misk Adventures"/>
        <genre-descr lang="ru" title="Приключения: Прочее"/>
        <genre-alt value="literature_adv" format="fb2.0"/>
        <genre-alt value="literature_men_advent" format="fb2.0"/>
      </subgenre>
      <subgenre value="child_adv">
        <genre-descr lang="en" title="Adventures for Kids"/>
        <genre-descr lang="ru" title="Детские Приключения"/>
      </subgenre>
    </subgenres>
  </genre>
  <genre value="children">
    <root-descr lang="en" genre-title="Children's" detailed="Fairy Tales, Fantasy, Detectives..."/>
    <root-descr lang="ru" genre-title="Детское" detailed="Сказки, фантастика, детективы..."/>
    <subgenres>
      <subgenre value="child_tale">
        <genre-descr lang="en" title="Fairy Tales"/>
        <genre-descr lang="ru" title="Сказки"/>
        <genre-alt value="child_3" format="fb2.0"/>
        <genre-alt value="literature_fairy" format="fb2.0"/>
      </subgenre>
      <subgenre value="child_verse">
        <genre-descr lang="en" title="Verses"/>
        <genre-descr lang="ru" title="Детские Стихи"/>
      </subgenre>
      <subgenre value="child_prose">
        <genre-descr lang="en" title="Prose for Kids"/>
        <genre-descr lang="ru" title="Детская Проза"/>
        <genre-alt value="child_4" format="fb2.0"/>
        <genre-alt value="child_9" format="fb2.0"/>
        <genre-alt value="child_history" format="fb2.0"/>
        <genre-alt value="child_characters" format="fb2.0"/>
      </subgenre>
      <subgenre value="child_sf">
        <genre-descr lang="en" title="Science Fiction for Kids"/>
        <genre-descr lang="ru" title="Детская Фантастика"/>
      </subgenre>
      <subgenre value="child_det">
        <genre-descr lang="en" title="Detectives and Thrillers"/>
        <genre-descr lang="ru" title="Детские Остросюжетные"/>
      </subgenre>
      <subgenre value="child_adv">
        <genre-descr lang="en" title="Adventures for Kids"/>
        <genre-descr lang="ru" title="Детские Приключения"/>
        <genre-alt value="teens_history" format="fb2.0"/>
        <genre-alt value="teens_series" format="fb2.0"/>
      </subgenre>
      <subgenre value="child_education">
        <genre-descr lang="en" title="Education for Kids"/>
        <genre-descr lang="ru" title="Детская Образовательная литература"/>
        <genre-alt value="child_edu" format="fb2.0"/>
        <genre-alt value="child_nature" format="fb2.0"/>
      </subgenre>
      <subgenre value="children">
        <genre-descr lang="en" title="For Kids: Misk"/>
        <genre-descr lang="ru" title="Детское: Прочее"/>
        <genre-alt value="child_art" format="fb2.0"/>
        <genre-alt value="child_obsessions" format="fb2.0"/>
        <genre-alt value="child_people" format="fb2.0"/>
        <genre-alt value="child_ref" format="fb2.0"/>
        <genre-alt value="child_series" format="fb2.0"/>
        <genre-alt value="child_sports" format="fb2.0"/>
      </subgenre>
    </subgenres>
  </genre>
  <genre value="poetry">
    <root-descr lang="en" genre-title="Poetry, Dramaturgy" detailed="Poetry, Dramaturgy"/>
    <root-descr lang="ru" genre-title="Поэзия, Драматургия" detailed="Поэзия, драматургия"/>
    <subgenres>
      <subgenre value="poetry">
        <genre-descr lang="en" title="Poetry"/>
        <genre-descr lang="ru" title="Поэзия"/>
        <genre-alt value="literature_poetry" format="fb2.0"/>
      </subgenre>
      <subgenre value="dramaturgy">
        <genre-descr lang="en" title="Dramaturgy"/>
        <genre-descr lang="ru" title="Драматургия"/>
        <genre-alt value="performance" format="fb2.0"/>
      </subgenre>
      <subgenre value="humor_verse">
        <genre-descr lang="en" title="Humor Verses"/>
        <genre-descr lang="ru" title="Юмористические Стихи"/>
      </subgenre>
      <subgenre value="child_verse">
        <genre-descr lang="en" title="Verses"/>
        <genre-descr lang="ru" title="Детские Стихи"/>
      </subgenre>
    </subgenres>
  </genre>
  <genre value="antique">
    <root-descr lang="en" genre-title="Antique" detailed="Antique literature, Myths, Legends"/>
    <root-descr lang="ru" genre-title="Старинное" detailed="Античная литература, мифы, легенды"/>
    <subgenres>
      <subgenre value="antique_ant">
        <genre-descr lang="en" title="Antique Literature"/>
        <genre-descr lang="ru" title="Античная Литература"/>
      </subgenre>
      <subgenre value="antique_european">
        <genre-descr lang="en" title="European Literature"/>
        <genre-descr lang="ru" title="Европейская Старинная Литература"/>
      </subgenre>
      <subgenre value="antique_russian">
        <genre-descr lang="en" title="Antique Russian Literature"/>
        <genre-descr lang="ru" title="Древнерусская Литература"/>
      </subgenre>
      <subgenre value="antique_east">
        <genre-descr lang="en" title="Antique East Literature"/>
        <genre-descr lang="ru" title="Древневосточная Литература"/>
      </subgenre>
      <subgenre value="antique_myths">
        <genre-descr lang="en" title="Myths. Legends. Epos"/>
        <genre-descr lang="ru" title="Мифы. Легенды. Эпос"/>
        <genre-alt value="nonfiction_folklor" format="fb2.0"/>
      </subgenre>
      <subgenre value="antique">
        <genre-descr lang="en" title="Other Antique"/>
        <genre-descr lang="ru" title="Старинная Литература: Прочее"/>
      </subgenre>
    </subgenres>
  </genre>
  <genre value="science">
    <root-descr lang="en" genre-title="Science, Education" detailed="Physics, Philosophy, Psychology, Business literature"/>
    <root-descr lang="ru" genre-title="Наука, Образование" detailed="Физика, философия, психология, деловая литература"/>
    <subgenres>
      <subgenre value="sci_history">
        <genre-descr lang="en" title="History"/>
        <genre-descr lang="ru" title="История"/>
        <genre-alt value="history_africa" format="fb2.0"/>
        <genre-alt value="history_america" format="fb2.0"/>
        <genre-alt value="history_ancient" format="fb2.0"/>
        <genre-alt value="history_asia" format="fb2.0"/>
        <genre-alt value="history_australia" format="fb2.0"/>
        <genre-alt value="history_europe" format="fb2.0"/>
        <genre-alt value="history_study" format="fb2.0"/>
        <genre-alt value="history_jewish" format="fb2.0"/>
        <genre-alt value="history_middle_east" format="fb2.0"/>
        <genre-alt value="histor_military" format="fb2.0"/>
        <genre-alt value="history_military_science" format="fb2.0"/>
        <genre-alt value="history_russia" format="fb2.0"/>
        <genre-alt value="history_usa" format="fb2.0"/>
        <genre-alt value="history_world" format="fb2.0"/>
        <genre-alt value="nonfiction_antropology" format="fb2.0"/>
        <genre-alt value="science_archaeology" format="fb2.0"/>
        <genre-alt value="ref_genealogy" format="fb2.0"/>
        <genre-alt value="science_history_philosophy" format="fb2.0"/>
      </subgenre>
      <subgenre value="sci_psychology">
        <genre-descr lang="en" title="Psychology"/>
        <genre-descr lang="ru" title="Психология"/>
        <genre-alt value="health_mental" format="fb2.0"/>
        <genre-alt value="health_psy" format="fb2.0"/>
        <genre-alt value="science_behavioral_sciences" format="fb2.0"/>
        <genre-alt value="science_psy" format="fb2.0"/>
        <genre-alt value="teens_social" format="fb2.0"/>
      </subgenre>
      <subgenre value="sci_culture">
        <genre-descr lang="en" title="Cultural Science"/>
        <genre-descr lang="ru" title="Культурология"/>
      </subgenre>
      <subgenre value="sci_religion">
        <genre-descr lang="en" title="Religious Studies"/>
        <genre-descr lang="ru" title="Религиоведение"/>
      </subgenre>
      <subgenre value="sci_philosophy">
        <genre-descr lang="en" title="Philosophy"/>
        <genre-descr lang="ru" title="Философия"/>
        <genre-alt value="nonfiction_philosophy" format="fb2.0"/>
      </subgenre>
      <subgenre value="sci_politics">
        <genre-descr lang="en" title="Politics"/>
        <genre-descr lang="ru" title="Политика"/>
      </subgenre>
      <subgenre value="sci_business">
        <genre-descr lang="en" title="Business literature"/>
        <genre-descr lang="ru" title="Деловая литература"/>
        <genre-alt value="biz_accounting" format="fb2.0"/>
        <genre-alt value="biz_life" format="fb2.0"/>
        <genre-alt value="biz_careers" format="fb2.0"/>
        <genre-alt value="biz_economics" format="fb2.0"/>
        <genre-alt value="biz_finance" format="fb2.0"/>
        <genre-alt value="biz_international" format="fb2.0"/>
        <genre-alt value="biz_professions" format="fb2.0"/>
        <genre-alt value="biz_investing" format="fb2.0"/>
        <genre-alt value="biz_management" format="fb2.0"/>
        <genre-alt value="biz_sales" format="fb2.0"/>
        <genre-alt value="biz_personal_fin" format="fb2.0"/>
        <genre-alt value="biz_ref" format="fb2.0"/>
        <genre-alt value="biz_small_biz" format="fb2.0"/>
        <genre-alt value="professional_finance" format="fb2.0"/>
        <genre-alt value="professional_management" format="fb2.0"/>
        <genre-alt value="ref_edu" format="fb2.0"/>
      </subgenre>
      <subgenre value="sci_juris">
        <genre-descr lang="en" title="Jurisprudence"/>
        <genre-descr lang="ru" title="Юриспруденция"/>
        <genre-alt value="nonfiction_law" format="fb2.0"/>
        <genre-alt value="professional_law" format="fb2.0"/>
      </subgenre>
      <subgenre value="sci_linguistic">
        <genre-descr lang="en" title="Linguistics"/>
        <genre-descr lang="ru" title="Языкознание"/>
      </subgenre>
      <subgenre value="sci_medicine">
        <genre-descr lang="en" title="Medicine"/>
        <genre-descr lang="ru" title="Медицина"/>
        <genre-alt value="health_aging" format="fb2.0"/>
        <genre-alt value="health_alt_medicine" format="fb2.0"/>
        <genre-alt value="health_cancer" format="fb2.0"/>
        <genre-alt value="professional_medical" format="fb2.0"/>
        <genre-alt value="science_medicine" format="fb2.0"/>
      </subgenre>
      <subgenre value="sci_phys">
        <genre-descr lang="en" title="Physics"/>
        <genre-descr lang="ru" title="Физика"/>
        <genre-alt value="science_physics" format="fb2.0"/>
      </subgenre>
      <subgenre value="sci_math">
        <genre-descr lang="en" title="Mathematics"/>
        <genre-descr lang="ru" title="Математика"/>
        <genre-alt value="science_math" format="fb2.0"/>
      </subgenre>
      <subgenre value="sci_chem">
        <genre-descr lang="en" title="Chemistry"/>
        <genre-descr lang="ru" title="Химия"/>
        <genre-alt value="science_chemistry" format="fb2.0"/>
      </subgenre>
      <subgenre value="sci_biology">
        <genre-descr lang="en" title="Biology"/>
        <genre-descr lang="ru" title="Биология"/>
        <genre-alt value="outdoors_birdwatching" format="fb2.0"/>
        <genre-alt value="outdoors_ecology" format="fb2.0"/>
        <genre-alt value="outdoors_ecosystems" format="fb2.0"/>
        <genre-alt value="outdoors_env" format="fb2.0"/>
        <genre-alt value="outdoors_fauna" format="fb2.0"/>
        <genre-alt value="outdoors_flora" format="fb2.0"/>
        <genre-alt value="outdoors_nature_writing" format="fb2.0"/>
        <genre-alt value="outdoors_ref" format="fb2.0"/>
        <genre-alt value="science_biolog" format="fb2.0"/>
      </subgenre>
      <subgenre value="sci_tech">
        <genre-descr lang="en" title="Technical"/>
        <genre-descr lang="ru" title="Технические"/>
        <genre-alt value="professional_enginering" format="fb2.0"/>
        <genre-alt value="professional_sci" format="fb2.0"/>
        <genre-alt value="science_technology" format="fb2.0"/>
      </subgenre>
      <subgenre value="science">
        <genre-descr lang="en" title="Misk Science, Education"/>
        <genre-descr lang="ru" title="Научно-образовательная: Прочее"/>
        <genre-alt value="nonfiction_edu" format="fb2.0"/>
        <genre-alt value="nonfiction_gov" format="fb2.0"/>
        <genre-alt value="nonfiction_holidays" format="fb2.0"/>
        <genre-alt value="nonfiction_social_sci" format="fb2.0"/>
        <genre-alt value="nonfiction_ethnology" format="fb2.0"/>
        <genre-alt value="nonfiction_gender" format="fb2.0"/>
        <genre-alt value="nonfiction_gerontology" format="fb2.0"/>
        <genre-alt value="nonfiction_hum_geogr" format="fb2.0"/>
        <genre-alt value="nonfiction_methodology" format="fb2.0"/>
        <genre-alt value="nonfiction_research" format="fb2.0"/>
        <genre-alt value="nonfiction_social_work" format="fb2.0"/>
        <genre-alt value="nonfiction_sociology" format="fb2.0"/>
        <genre-alt value="nonfiction_spec_group" format="fb2.0"/>
        <genre-alt value="nonfiction_stat" format="fb2.0"/>
        <genre-alt value="outdoors_resources" format="fb2.0"/>
        <genre-alt value="professional_edu" format="fb2.0"/>
        <genre-alt value="science_agri" format="fb2.0"/>
        <genre-alt value="science_astronomy" format="fb2.0"/>
        <genre-alt value="science_earth" format="fb2.0"/>
        <genre-alt value="science_edu" format="fb2.0"/>
        <genre-alt value="science_evolution" format="fb2.0"/>
        <genre-alt value="science_measurement" format="fb2.0"/>
        <genre-alt value="science_eco" format="fb2.0"/>
        <genre-alt value="science_ref" format="fb2.0"/>
        <genre-alt value="teens_tech" format="fb2.0"/>
      </subgenre>
    </subgenres>
  </genre>
  <genre value="computers">
    <root-descr lang="en" genre-title="Computers" detailed="Internet, Programming, Hardware"/>
    <root-descr lang="ru" genre-title="Компьютеры" detailed="Интернет, программирование, железо"/>
    <subgenres>
      <subgenre value="comp_www">
        <genre-descr lang="en" title="Internet"/>
        <genre-descr lang="ru" title="Интернет"/>
      </subgenre>
      <subgenre value="comp_programming">
        <genre-descr lang="en" title="Programming"/>
        <genre-descr lang="ru" title="Программирование"/>
      </subgenre>
      <subgenre value="comp_hard">
        <genre-descr lang="en" title="Hardware"/>
        <genre-descr lang="ru" title="Компьютерное Железо"/>
        <genre-alt value="comp_hardware" format="fb2.0"/>
      </subgenre>
      <subgenre value="comp_soft">
        <genre-descr lang="en" title="Software"/>
        <genre-descr lang="ru" title="Программы"/>
        <genre-alt value="comp_software" format="fb2.0"/>
      </subgenre>
      <subgenre value="comp_db">
        <genre-descr lang="en" title="Databases"/>
        <genre-descr lang="ru" title="Базы Данных"/>
      </subgenre>
      <subgenre value="comp_osnet">
        <genre-descr lang="en" title="OS and Networking"/>
        <genre-descr lang="ru" title="ОС и Сети"/>
        <genre-alt value="comp_microsoft" format="fb2.0"/>
        <genre-alt value="comp_networking" format="fb2.0"/>
        <genre-alt value="comp_os" format="fb2.0"/>
      </subgenre>
      <subgenre value="computers">
        <genre-descr lang="en" title="Computers: Misk"/>
        <genre-descr lang="ru" title="Компьютеры: Прочее"/>
        <genre-alt value="child_computers" format="fb2.0"/>
        <genre-alt value="compusers" format="fb2.0"/>
        <genre-alt value="comp_office" format="fb2.0"/>
        <genre-alt value="comp_cert" format="fb2.0"/>
        <genre-alt value="comp_games" format="fb2.0"/>
        <genre-alt value="comp_sci" format="fb2.0"/>
        <genre-alt value="comp_biz" format="fb2.0"/>
        <genre-alt value="comp_graph" format="fb2.0"/>
      </subgenre>
    </subgenres>
  </genre>
  <genre value="reference">
    <root-descr lang="en" genre-title="Reference" detailed="Reference, Encyclopedias, Dictionaries"/>
    <root-descr lang="ru" genre-title="Справочники" detailed="Справочники, энциклопедии, словари"/>
    <subgenres>
      <subgenre value="ref_encyc">
        <genre-descr lang="en" title="Encyclopedias"/>
        <genre-descr lang="ru" title="Энциклопедии"/>
        <genre-alt value="ref_encyclopedia" format="fb2.0"/>
      </subgenre>
      <subgenre value="ref_dict">
        <genre-descr lang="en" title="Dictionaries"/>
        <genre-descr lang="ru" title="Словари"/>
        <genre-alt value="ref_dict" format="fb2.0"/>
      </subgenre>
      <subgenre value="ref_ref">
        <genre-descr lang="en" title="Reference"/>
        <genre-descr lang="ru" title="Справочники"/>
        <genre-alt value="ref_almanacs" format="fb2.0"/>
        <genre-alt value="ref_careers" format="fb2.0"/>
        <genre-alt value="ref_catalogs" format="fb2.0"/>
        <genre-alt value="ref_cons_guides" format="fb2.0"/>
        <genre-alt value="ref_study_guides" format="fb2.0"/>
      </subgenre>
      <subgenre value="ref_guide">
        <genre-descr lang="en" title="Guidebooks"/>
        <genre-descr lang="ru" title="Руководства"/>
        <genre-alt value="outdoors_field_guides" format="fb2.0"/>
      </subgenre>
      <subgenre value="reference">
        <genre-descr lang="en" title="Misk References"/>
        <genre-descr lang="ru" title="Справочная Литература: Прочее"/>
        <genre-alt value="nonfiction_ref" format="fb2.0"/>
        <genre-alt value="family_ref" format="fb2.0"/>
        <genre-alt value="references" format="fb2.0"/>
        <genre-alt value="ref_etiquette" format="fb2.0"/>
        <genre-alt value="ref_langs" format="fb2.0"/>
        <genre-alt value="ref_fun" format="fb2.0"/>
        <genre-alt value="ref_books" format="fb2.0"/>
        <genre-alt value="ref_quotations" format="fb2.0"/>
        <genre-alt value="ref_words" format="fb2.0"/>
        <genre-alt value="teens_ref" format="fb2.0"/>
      </subgenre>
    </subgenres>
  </genre>
  <genre value="nonfiction">
    <root-descr lang="en" genre-title="Nonfiction" detailed="Biography, Memoirs, Publicism"/>
    <root-descr lang="ru" genre-title="Документальное" detailed="Биографии, мемуары, публицистика"/>
    <subgenres>
      <subgenre value="nonf_biography">
        <genre-descr lang="en" title="Biography and Memoirs"/>
        <genre-descr lang="ru" title="Биографии и Мемуары"/>
        <genre-alt value="people" format="fb2.0"/>
        <genre-alt value="biography" format="fb2.0"/>
        <genre-alt value="biogr_arts" format="fb2.0"/>
        <genre-alt value="biogr_ethnic" format="fb2.0"/>
        <genre-alt value="biogr_family" format="fb2.0"/>
        <genre-alt value="biogr_historical" format="fb2.0"/>
        <genre-alt value="biogr_leaders" format="fb2.0"/>
        <genre-alt value="biogr_professionals" format="fb2.0"/>
        <genre-alt value="biogr_sports" format="fb2.0"/>
        <genre-alt value="biogr_travel" format="fb2.0"/>
        <genre-alt value="biz_beogr" format="fb2.0"/>
        <genre-alt value="gay_biogr" format="fb2.0"/>
        <genre-alt value="history_gay" format="fb2.0"/>
        <genre-alt value="literature_letters" format="fb2.0"/>
        <genre-alt value="teens_beogr" format="fb2.0"/>
      </subgenre>
      <subgenre value="nonf_publicism">
        <genre-descr lang="en" title="Publicism"/>
        <genre-descr lang="ru" title="Публицистика"/>
      </subgenre>
      <subgenre value="nonf_criticism">
        <genre-descr lang="en" title="Criticism"/>
        <genre-descr lang="ru" title="Критика"/>
      </subgenre>
      <subgenre value="nonfiction">
        <genre-descr lang="en" title="Misk Nonfiction"/>
        <genre-descr lang="ru" title="Документальное: Прочее"/>
        <genre-alt value="gay_nonfiction" format="fb2.0"/>
        <genre-alt value="nonfiction_avto" format="fb2.0"/>
        <genre-alt value="nonfiction_crime" format="fb2.0"/>
        <genre-alt value="nonfiction_events" format="fb2.0"/>
        <genre-alt value="nonfiction_politics" format="fb2.0"/>
        <genre-alt value="nonfiction_traditions" format="fb2.0"/>
        <genre-alt value="nonfiction_demography" format="fb2.0"/>
        <genre-alt value="nonfiction_racism" format="fb2.0"/>
        <genre-alt value="nonfiction_emigration" format="fb2.0"/>
        <genre-alt value="nonfiction_philantropy" format="fb2.0"/>
        <genre-alt value="nonfiction_transportation" format="fb2.0"/>
        <genre-alt value="nonfiction_true_accounts" format="fb2.0"/>
        <genre-alt value="nonfiction_urban" format="fb2.0"/>
        <genre-alt value="nonfiction_women" format="fb2.0"/>
        <genre-alt value="outdoors_conservation" format="fb2.0"/>
      </subgenre>
      <subgenre value="design">
        <genre-descr lang="en" title="Art, Design"/>
        <genre-descr lang="ru" title="Искусство, Дизайн"/>
        <genre-alt value="architecture" format="fb2.0"/>
        <genre-alt value="art" format="fb2.0"/>
        <genre-alt value="art_instr" format="fb2.0"/>
        <genre-alt value="artists" format="fb2.0"/>
        <genre-alt value="fashion" format="fb2.0"/>
        <genre-alt value="graph_design" format="fb2.0"/>
        <genre-alt value="photography" format="fb2.0"/>
      </subgenre>
    </subgenres>
  </genre>
  <genre value="religion">
    <root-descr lang="en" genre-title="Religion" detailed="Religion, Esoterics"/>
    <root-descr lang="ru" genre-title="Религия" detailed="Религия, эзотерика"/>
    <subgenres>
      <subgenre value="religion_rel">
        <genre-descr lang="en" title="Religion"/>
        <genre-descr lang="ru" title="Религия"/>
        <genre-alt value="child_religion" format="fb2.0"/>
        <genre-alt value="chris_bibles" format="fb2.0"/>
        <genre-alt value="chris_pravoslavie" format="fb2.0"/>
        <genre-alt value="chris_catholicism" format="fb2.0"/>
        <genre-alt value="chris_living" format="fb2.0"/>
        <genre-alt value="chris_history" format="fb2.0"/>
        <genre-alt value="chris_clergy" format="fb2.0"/>
        <genre-alt value="chris_edu" format="fb2.0"/>
        <genre-alt value="chris_evangelism" format="fb2.0"/>
        <genre-alt value="chris_fiction" format="fb2.0"/>
        <genre-alt value="chris_holidays" format="fb2.0"/>
        <genre-alt value="chris_jesus" format="fb2.0"/>
        <genre-alt value="chris_mormonism" format="fb2.0"/>
        <genre-alt value="chris_orthodoxy" format="fb2.0"/>
        <genre-alt value="outdoors_conservation" format="fb2.0"/>
        <genre-alt value="chris_protestantism" format="fb2.0"/>
        <genre-alt value="chris_ref" format="fb2.0"/>
        <genre-alt value="chris_theology" format="fb2.0"/>
        <genre-alt value="chris_devotion" format="fb2.0"/>
        <genre-alt value="literature_religion" format="fb2.0"/>
        <genre-alt value="religion" format="fb2.0"/>
        <genre-alt value="religion_bibles" format="fb2.0"/>
        <genre-alt value="Christianity" format="fb2.0"/>
        <genre-alt value="religion_fiction" format="fb2.0"/>
        <genre-alt value="religion_new_age" format="fb2.0"/>
        <genre-alt value="religion_religious_studies" format="fb2.0"/>
        <genre-alt value="romance_religion" format="fb2.0"/>
        <genre-alt value="teens_religion" format="fb2.0"/>
      </subgenre>
      <subgenre value="religion_esoterics">
        <genre-descr lang="en" title="Esoterics"/>
        <genre-descr lang="ru" title="Эзотерика"/>
        <genre-alt value="religion_occult" format="fb2.0"/>
        <genre-alt value="religion_spirituality" format="fb2.0"/>
      </subgenre>
      <subgenre value="religion_self">
        <genre-descr lang="en" title="Self-perfection"/>
        <genre-descr lang="ru" title="Самосовершенствование"/>
      </subgenre>
      <subgenre value="religion">
        <genre-descr lang="en" title="Religion: Other"/>
        <genre-descr lang="ru" title="Религия и духовность: Прочее"/>
        <genre-alt value="religion_east" format="fb2.0"/>
        <genre-alt value="religion_buddhism" format="fb2.0"/>
        <genre-alt value="religion_earth" format="fb2.0"/>
        <genre-alt value="religion_hinduism" format="fb2.0"/>
        <genre-alt value="religion_islam" format="fb2.0"/>
        <genre-alt value="religion_judaism" format="fb2.0"/>
        <genre-alt value="religion_other" format="fb2.0"/>
      </subgenre>
      <subgenre value="sci_religion">
        <genre-descr lang="en" title="Religious Studies"/>
        <genre-descr lang="ru" title="Религиоведение"/>
      </subgenre>
    </subgenres>
  </genre>
  <genre value="humor">
    <root-descr lang="en" genre-title="Humor" detailed="Prose, Verses, Anecdote"/>
    <root-descr lang="ru" genre-title="Юмор" detailed="Проза, стихи, анекдоты"/>
    <subgenres>
      <subgenre value="humor_anecdote">
        <genre-descr lang="en" title="Anecdote"/>
        <genre-descr lang="ru" title="Анекдоты"/>
      </subgenre>
      <subgenre value="humor_prose">
        <genre-descr lang="en" title="Humor Prose"/>
        <genre-descr lang="ru" title="Юмористическая Проза"/>
      </subgenre>
      <subgenre value="humor_verse">
        <genre-descr lang="en" title="Humor Verses"/>
        <genre-descr lang="ru" title="Юмористические Стихи"/>
      </subgenre>
      <subgenre value="humor">
        <genre-descr lang="en" title="Misk Humor"/>
        <genre-descr lang="ru" title="Юмор: Прочее"/>
        <genre-alt value="family_humor" format="fb2.0"/>
      </subgenre>
    </subgenres>
  </genre>
  <genre value="home">
    <root-descr lang="en" genre-title="Home, Family" detailed="Cooking, Pets, Hobby"/>
    <root-descr lang="ru" genre-title="Дом, Семья" detailed="Кулинария, домашние животные, хобби"/>
    <subgenres>
      <subgenre value="home_cooking">
        <genre-descr lang="en" title="Cooking"/>
        <genre-descr lang="ru" title="Кулинария"/>
        <genre-alt value="cooking" format="fb2.0"/>
        <genre-alt value="cook_baking" format="fb2.0"/>
        <genre-alt value="cook_can" format="fb2.0"/>
        <genre-alt value="cook_art" format="fb2.0"/>
        <genre-alt value="cook_drink" format="fb2.0"/>
        <genre-alt value="cook_gastronomy" format="fb2.0"/>
        <genre-alt value="cook_meals" format="fb2.0"/>
        <genre-alt value="cook_natura" format="fb2.0"/>
        <genre-alt value="cook_outdoor" format="fb2.0"/>
        <genre-alt value="cook_pro" format="fb2.0"/>
        <genre-alt value="cook_quick" format="fb2.0"/>
        <genre-alt value="cook_ref" format="fb2.0"/>
        <genre-alt value="cook_regional" format="fb2.0"/>
        <genre-alt value="cook_appliances" format="fb2.0"/>
        <genre-alt value="cook_diet" format="fb2.0"/>
        <genre-alt value="cook_spec" format="fb2.0"/>
        <genre-alt value="cook_veget" format="fb2.0"/>
        <genre-alt value="health_diets" format="fb2.0"/>
      </subgenre>
      <subgenre value="home_pets">
        <genre-descr lang="en" title="Pets"/>
        <genre-descr lang="ru" title="Домашние Животные"/>
      </subgenre>
      <subgenre value="home_crafts">
        <genre-descr lang="en" title="Hobbies and Crafts"/>
        <genre-descr lang="ru" title="Хобби, Ремесла"/>
        <genre-alt value="home_collect" format="fb2.0"/>
        <genre-alt value="outdoors_hiking" format="fb2.0"/>
        <genre-alt value="outdoors_hunt_fish" format="fb2.0"/>
      </subgenre>
      <subgenre value="home_entertain">
        <genre-descr lang="en" title="Entertaining"/>
        <genre-descr lang="ru" title="Развлечения"/>
        <genre-alt value="entertainment" format="fb2.0"/>
        <genre-alt value="entert_comics" format="fb2.0"/>
        <genre-alt value="entert_games" format="fb2.0"/>
        <genre-alt value="entert_humor" format="fb2.0"/>
        <genre-alt value="entert_movies" format="fb2.0"/>
        <genre-alt value="entert_music" format="fb2.0"/>
        <genre-alt value="nonfiction_pop_culture" format="fb2.0"/>
        <genre-alt value="entert_radio" format="fb2.0"/>
        <genre-alt value="entert_tv" format="fb2.0"/>
      </subgenre>
      <subgenre value="home_health">
        <genre-descr lang="en" title="Health"/>
        <genre-descr lang="ru" title="Здоровье"/>
        <genre-alt value="health" format="fb2.0"/>
        <genre-alt value="health_beauty" format="fb2.0"/>
        <genre-alt value="family_health" format="fb2.0"/>
        <genre-alt value="family_fertility" format="fb2.0"/>
        <genre-alt value="family_parenting" format="fb2.0"/>
        <genre-alt value="family_pregnancy" format="fb2.0"/>
        <genre-alt value="family_special_needs" format="fb2.0"/>
        <genre-alt value="health_death" format="fb2.0"/>
        <genre-alt value="health_dideases" format="fb2.0"/>
        <genre-alt value="health_fitness" format="fb2.0"/>
        <genre-alt value="health_men" format="fb2.0"/>
        <genre-alt value="health_nutrition" format="fb2.0"/>
        <genre-alt value="health_personal" format="fb2.0"/>
        <genre-alt value="health_recovery" format="fb2.0"/>
        <genre-alt value="health_ref" format="fb2.0"/>
        <genre-alt value="health_first_aid" format="fb2.0"/>
        <genre-alt value="health_self_help" format="fb2.0"/>
        <genre-alt value="health_women" format="fb2.0"/>
      </subgenre>
      <subgenre value="home_garden">
        <genre-descr lang="en" title="Garden"/>
        <genre-descr lang="ru" title="Сад и Огород"/>
      </subgenre>
      <subgenre value="home_diy">
        <genre-descr lang="en" title="Do it yourself"/>
        <genre-descr lang="ru" title="Сделай Сам"/>
        <genre-alt value="home_expert" format="fb2.0"/>
        <genre-alt value="home_design" format="fb2.0"/>
        <genre-alt value="home_howto" format="fb2.0"/>
        <genre-alt value="home_interior_design" format="fb2.0"/>
      </subgenre>
      <subgenre value="home_sport">
        <genre-descr lang="en" title="Sports"/>
        <genre-descr lang="ru" title="Спорт"/>
        <genre-alt value="literature_sports" format="fb2.0"/>
        <genre-alt value="outdoors_outdoor_recreation" format="fb2.0"/>
        <genre-alt value="outdoors_survive" format="fb2.0"/>
        <genre-alt value="sport" format="fb2.0"/>
        <genre-alt value="teens_health" format="fb2.0"/>
        <genre-alt value="teens_school_sports" format="fb2.0"/>
      </subgenre>
      <subgenre value="home_sex">
        <genre-descr lang="en" title="Erotica, Sex"/>
        <genre-descr lang="ru" title="Эротика, Секс"/>
        <genre-alt value="health_sex" format="fb2.0"/>
        <genre-alt value="nonfiction_pornography" format="fb2.0"/>
      </subgenre>
      <subgenre value="home">
        <genre-descr lang="en" title="Home: Other"/>
        <genre-descr lang="ru" title="Дом и Семья: Прочее"/>
        <genre-alt value="gay_parenting" format="fb2.0"/>
        <genre-alt value="home_cottage" format="fb2.0"/>
        <genre-alt value="home_weddings" format="fb2.0"/>
        <genre-alt value="family" format="fb2.0"/>
        <genre-alt value="family_adoption" format="fb2.0"/>
        <genre-alt value="family_aging_parents" format="fb2.0"/>
        <genre-alt value="family_edu" format="fb2.0"/>
        <genre-alt value="family_activities" format="fb2.0"/>
        <genre-alt value="family_relations" format="fb2.0"/>
        <genre-alt value="family_lit_guide" format="fb2.0"/>
        <genre-alt value="women_divorce" format="fb2.0"/>
        <genre-alt value="women_domestic" format="fb2.0"/>
        <genre-alt value="women_child" format="fb2.0"/>
        <genre-alt value="women_single" format="fb2.0"/>
      </subgenre>
    </subgenres>
  </genre>
</fbgenrestransfer>
'''




from .epub2 import Epub2
from .utils import xstr

class Epub3(Epub2):
    ####### Getters (override) #######
    def get_title(self):
        node = self._get_title_node()
        if node is not None:
            return xstr(node.text)
            
    def get_author_list(self):
        result = []
        node_list = self._get_person_node_list('aut')
        for node in node_list: result.append(xstr(node.text))
        return result

    def get_translator_list(self):
        result = []
        node_list = self._get_person_node_list('trl')
        for node in node_list: result.append(xstr(node.text))
        return result

    def get_cover_data(self):
        media_type = None
        href = None
        data = None
        node = self._get('opf:manifest/opf:item[@properties="cover-image"]')
        if node is not None:
            if 'media-type' in node.attrib: media_type = node.attrib['media-type']
            if 'href'in node.attrib: href = node.attrib['href']
            if href: data = self._get_file_content(self.content_root + href)
        else:
            return super().get_cover_data()
        return (href, media_type, data)

    ####### Setters (override) #######
    def set_title(self, title):
        node = self._get_title_node()
        if node is not None:
            node.text = title

    def set_author_list(self, author_list):
        self._set_person_list(author_list, 'aut')

    def set_translator_list(self, translator_list):
        self._set_person_list(translator_list, 'trl')

    def set_cover_data(self, href, media_type, data):
        (href, _, _) = self.get_cover_data()
        if href and data:
            self.cover_href = href
            self.cover_data = data

    ####### Service methods #######
    def _set_person_list(self, person_list, role):
        node_list = self._get_person_node_list(role)
        for node in node_list:
            if 'id' in node.attrib:
                refine_list = self._get_all('opf:metadata/opf:meta[@refines="#{0}"]'.format(node.attrib['id']))
                for refine in refine_list: refine.getparent().remove(refine)
            node.getparent().remove(node)

        meta_node = self._get('opf:metadata')
        index = 1 
        for person in person_list:
            if person:
                node = self._sub_element(meta_node, 'dc:creator')
                node.text = person
                node.attrib['id'] = '{0}{1:02d}'.format(role, index)
                node.tail = '\n'
                refine = self._sub_element(meta_node, 'opf:meta')
                refine.attrib['refines'] = '#{0}{1:02d}'.format(role, index)
                refine.attrib['scheme']='marc:relators'
                refine.attrib['property'] = 'role'
                refine.text = role
                refine.tail = '\n'

                index += 1

    def _get_title_node(self):
        node_list = self._get_all('opf:metadata/dc:title')
        for node in node_list:
            refines_list = self._get_all('opf:metadata/opf:meta[@refines="#{0}" and @property="title-type"]'.format(self._get_element_id(node)))
            if len(refines_list) == 0:
                return node
            else:
                for refines in refines_list:
                    if refines.text == 'main':
                        return node
    
    def _get_person_node_list(self, role):
        result_list = []
        node_list = self._get_all('opf:metadata/dc:creator')
        for node in node_list:
            refines_list = self._get_all('opf:metadata/opf:meta[@refines="#{0}" and @property="role"]'.format(self._get_element_id(node)))
            if len(refines_list) == 0 and role == 'aut':
                result_list.append(node)
            else:
                for refines in refines_list:
                    if refines.text == role:
                        result_list.append(node)
                        break

        return result_list

    def _get_element_refines(self, id, property):
        value = self._get('opf:metadata/opf:meta[@refines="#{0}" and @property="{1}"]'.format(id, property))
        if value is not None:
            return xstr(value.text)
        return ''

    def _get_element_id(self, e):
        return e.attrib['id'] if 'id' in e.attrib else ''


from .metadata import Metadata
from .exceptions import BadFormat, UnknownFormatException
from .fb2 import Fb2
from .epub2 import Epub2
from .epub3 import Epub3
from .utils import get_file_creation_time, get_file_modified_time

__all__ = ['get_metadata', 'set_metadata', 'Metadata', 'get_filename_from_pattern']


def _get_ebook(file):
    ebook = None
    
    if file.lower().endswith(('.fb2', '.zip')):
        ebook = Fb2(file)

    elif file.lower().endswith('.epub'):
        ebook = Epub2(file)
        if ebook.version[:1] == '3':
            ebook = Epub3(file)
    else:
       raise UnknownFormatException 

    return ebook


def get_metadata(file):
    ebook = _get_ebook(file)
    meta = Metadata()

    meta.identifier = ebook.get_identifier()
    meta.title = ebook.get_title()
    meta.author_list = ebook.get_author_list()
    meta.series = ebook.get_series()
    meta.series_index = ebook.get_series_index()
    meta.lang = ebook.get_lang()
    meta.description = ebook.get_description()
    meta.tag_list = ebook.get_tag_list()
    meta.translator_list = ebook.get_translator_list()
    (meta.cover_file_name, meta.cover_media_type, meta.cover_image_data) = ebook.get_cover_data()
    meta.format = ebook.get_format()
    meta.format_version = ebook.get_format_version()
    meta.file = file
    meta.file_created = get_file_creation_time(file)
    meta.file_modified = get_file_modified_time(file)

    # Get publish info for FB2
    if meta.format == 'fb2':
        meta.publish_info.title = ebook.get_publish_title()
        meta.publish_info.publisher = ebook.get_publish_publisher()
        meta.publish_info.city = ebook.get_publish_city()
        meta.publish_info.year = ebook.get_publish_year()
        meta.publish_info.series = ebook.get_publish_series()
        meta.publish_info.series_index = ebook.get_publish_series_index()
        meta.publish_info.isbn = ebook.get_publish_isbn()
   
    return meta

def set_metadata(file, meta):
    ebook = _get_ebook(file)

    ebook.set_title(meta.title)
    ebook.set_author_list(meta.author_list)
    ebook.set_series(meta.series)
    ebook.set_series_index(meta.series_index)
    ebook.set_lang(meta.lang)
    ebook.set_tag_list(meta.tag_list)
    ebook.set_translator_list(meta.translator_list)
    ebook.set_cover_data(meta.cover_file_name, meta.cover_media_type, meta.cover_image_data)

    # Set publish info for FB2
    if meta.format == 'fb2':
        ebook.set_publish_title(meta.publish_info.title)
        ebook.set_publish_publisher(meta.publish_info.publisher)
        ebook.set_publish_city(meta.publish_info.city) 
        ebook.set_publish_year(meta.publish_info.year)
        ebook.set_publish_series(meta.publish_info.series)
        ebook.set_publish_series_index(meta.publish_info.series_index)
        ebook.set_publish_isbn(meta.publish_info.isbn)
   
    ebook.save()






class Error(Exception):

    def __init__(self, msg=''):
        Exception.__init__(self, msg)
        self.msg = msg

    def __repr__(self):
        ret = 'ebookmeta.{} {}'.format(self.__class__.__name__, self.msg)
        return ret.strip()

    __str__ = __repr__


class BadFormat(Error):
    pass


class BadLanguage(Error):
    pass


class WriteEpubException(Error):
    pass


class UnknownFormatException(Error):
    pass


class BadEpubVersionException(Error):
    pass



from setuptools import setup


setup(
    name='ebookmeta',
    version='1.2.1',
    author='Dmitrii Korpushov',
    author_email='dnkorpushov@gmail.com',
    packages=['ebookmeta'],
    url='https://github.com/dnkorpushov/ebookmeta',
    license='MIT',
    description='Read/write ebook metadata for fb2, epub2 and epub3 files',
    keywords=['ebook', 'epub', 'metadata', 'fb2'],
    classifiers=[
        'Operating System :: OS Independent',
        'Topic :: Software Development',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],

    install_requires=['lxml']
)



from lxml import etree
import base64
from io import BytesIO

from .utils import xstr
from .myzipfile import ZipFile, is_zipfile

ns_map = {
    'fb': 'http://www.gribuser.ru/xml/fictionbook/2.0',
    'l': 'http://www.w3.org/1999/xlink'
}

class Fb2():
    def __init__(self, file):
        self.file = file
        self.tree = None
        self.encoding = None
        self.zip_file_info = None

        if is_zipfile(self.file):
            zipfile = ZipFile(self.file)
            for info in zipfile.infolist():
                if info.filename.lower().endswith('.fb2'):
                    self.zip_file_info = info
                    break
            
            if self.zip_file_info:
                content = zipfile.read(self.zip_file_info)
                self.tree = etree.parse(BytesIO(content), parser=etree.XMLParser(recover=True))
                self.encoding = self.tree.docinfo.encoding
                zipfile.close()
        else:
            self.tree = etree.parse(self.file, parser=etree.XMLParser(recover=True))
            self.encoding = self.tree.docinfo.encoding

    ######## Getters ########
    def get_title(self):
        return xstr(self._get('//fb:description/fb:title-info/fb:book-title/text()'))

    def get_author_list(self):
        node_list = self._get_all('//fb:description/fb:title-info/fb:author')
        result_list = []
        for node in node_list:
            person = self._get_person(node)
            result_list.append(person)
        return result_list
    
    def get_series(self):
         return xstr(self._get('//fb:description/fb:title-info/fb:sequence/@name'))

    def get_series_index(self):
        return xstr(self._get('//fb:description/fb:title-info/fb:sequence/@number'))

    def get_lang(self):
        return xstr(self._get('//fb:description/fb:title-info/fb:lang/text()'))
    
    def get_tag_list(self):
        tag_list = []
        node_list = self._get_all('//fb:description/fb:title-info/fb:genre')
        for n in node_list:
            tag_list.append(xstr(n.text))
        return tag_list

    def get_description(self):
        content = self._get('//fb:description/fb:title-info/fb:annotation')
        if content is not None:
            return ''.join(content.itertext())

    def get_translator_list(self):
        node_list = self._get_all('//fb:description/fb:title-info/fb:translator')
        result_list = []
        for node in node_list:
            person = self._get_person(node)
            result_list.append(person)
        return result_list

    def get_format(self):
        return 'fb2'
    
    def get_format_version(self):
        return '2.0'

    def get_identifier(self):
        return xstr(self._get('//fb:description/fb:document-info/fb:id/text()'))

    def get_cover_data(self):
        media_type = None
        href = None
        data = None

        href = self._get('//fb:description/fb:title-info/fb:coverpage/fb:image/@l:href')
        if href:
            href = href[1:] # Crop # symbol
            node = self._get('//fb:binary[@id="{0}"]'.format(href))
            if node is not None:
                if 'content-type' in node.attrib:
                    media_type = node.attrib['content-type']
                else:
                    if href.lower().endswith(('.jpeg', '.jpg')):
                        media_type = 'image/jpeg'
                    elif href.lower().endswith('.png'):
                        media_type = 'image/png'
                data = base64.b64decode(node.text.encode('ascii'))
        return (href, media_type, data)

    def get_publish_title(self):
        return xstr(self._get('//fb:description/fb:publish-info/fb:book-name/text()'))

    def get_publish_publisher(self):
        return xstr(self._get('//fb:description/fb:publish-info/fb:publisher/text()'))
    
    def get_publish_year(self):
        return xstr(self._get('//fb:description/fb:publish-info/fb:year/text()'))

    def get_publish_city(self):
        return xstr(self._get('//fb:description/fb:publish-info/fb:city/text()'))
    
    def get_publish_isbn(self):
        return xstr(self._get('//fb:description/fb:publish-info/fb:isbn/text()'))

    def get_publish_series(self):
         return xstr(self._get('//fb:description/fb:publish-info/fb:sequence/@name'))

    def get_publish_series_index(self):
        return xstr(self._get('//fb:description/fb:publish-info/fb:sequence/@number'))
    

    ######## Setters ########
    def set_title(self, title):
        node = self._get('//fb:description/fb:title-info/fb:book-title')
        if node is None:
            parent = self._get('//fb:description/fb:title-info')
            node = self._sub_element(parent, 'fb:book-title')
        node.text = title

    def set_author_list(self, author_list):
        node_list = self._get_all('//fb:description/fb:title-info/fb:author')
        for node in node_list: node.getparent().remove(node)
        parent = self._get('//fb:description/fb:title-info')
        for author in author_list:
            if author:
                node = self._sub_element(parent, 'fb:author')
                self._set_person(node, author)
       
    def set_series(self, series):
        node = self._get('//fb:description/fb:title-info/fb:sequence')
        if series:
            if node is None:
                parent = self._get('//fb:description/fb:title-info')
                node = self._sub_element(parent, 'fb:sequence')
            node.attrib['name'] = series
        else:
            if node is not None:
                node.getparent().remove(node)

    def set_series_index(self, series_index):
        node = self._get('//fb:description/fb:title-info/fb:sequence')
        if series_index:
            if node is None:
                parent = self._get('//fb:description/fb:title-info')
                node = self._sub_element(parent, 'fb:sequence')
            node.attrib['number'] = str(series_index)
        else:
            if node is not None:
                node.getparent().remove(node)

    def set_lang(self, lang):
        node = self._get('//fb:description/fb:title-info/fb:lang')
        if node is None:
            parent = self._get('//fb:description/fb:title-info')
            node = self._sub_element(parent, 'fb:lang')
        node.text = lang

    def set_tag_list(self, tag_list):
        node_list = self._get_all('//fb:description/fb:title-info/fb:genre')
        for node in node_list: node.getparent().remove(node)
        parent = self._get('//fb:description/fb:title-info')
        for tag in tag_list:
            if tag:
                node = self._sub_element(parent, 'fb:genre')
                node.text = tag

    def set_translator_list(self, translator_list):
        node_list = self._get_all('//fb:description/fb:title-info/fb:translator')
        for node in node_list: node.getparent().remove(node)
        parent = self._get('//fb:description/fb:title-info')
        for translator in translator_list:
            if translator:
                node = self._sub_element(parent, 'fb:translator')
                self._set_person(node, translator)

    def set_cover_data(self, href, media_type, data):
        old_href = self._get('//fb:description/fb:title-info/fb:coverpage/fb:image/@l:href')
        if old_href:
            href = old_href[1:] # Crop # symbol
        else:
            parent = self._get('//fb:description/fb:title-info')
            node = self._sub_element(parent, 'fb:coverpage')
            image_node = self._sub_element(node, 'fb:image')
            image_node.attrib[etree.QName('http://www.w3.org/1999/xlink', 'href')] = '#{}'.format(href)

        node = self._get('//fb:binary[@id="{0}"]'.format(href))
        if node is None:
            node = self._sub_element(self.tree.getroot(), 'fb:binary')
            node.attrib['id'] = href
            node.attrib['content-type'] = media_type
        
        node.text = base64.encodebytes(data)

    def set_publish_title(self, title):
        node = self._get('//fb:description/fb:publish-info/fb:book-name')
        if node is None:
            parent = self._get('//fb:description/fb:publish-info')
            if parent is None:
                descr_node = self._get('//fb:description')
                parent = self._sub_element(descr_node, 'fb:publish-info')
            node = self._sub_element(parent, 'fb:book-name')
        if title:
            node.text = title
        else:
            node.getparent().remove(node)

    def set_publish_publisher(self, publisher):
        node = self._get('//fb:description/fb:publish-info/fb:publisher')
        if node is None:
            parent = self._get('//fb:description/fb:publish-info')
            if parent is None:
                descr_node = self._get('//fb:description')
                parent = self._sub_element(descr_node, 'fb:publish-info')
            node = self._sub_element(parent, 'fb:publisher')
        if publisher:
            node.text = publisher
        else:
            node.getparent().remove(node)

    def set_publish_year(self, year):
        node = self._get('//fb:description/fb:publish-info/fb:year')
        if node is None:
            parent = self._get('//fb:description/fb:publish-info')
            if parent is None:
                descr_node = self._get('//fb:description')
                parent = self._sub_element(descr_node, 'fb:publish-info')
            node = self._sub_element(parent, 'fb:year')
        if year:
            node.text = year
        else:
           node.getparent().remove(node) 

    def set_publish_city(self, city):
        node = self._get('//fb:description/fb:publish-info/fb:city')
        if node is None:
            parent = self._get('//fb:description/fb:publish-info')
            if parent is None:
                descr_node = self._get('//fb:description')
                parent = self._sub_element(descr_node, 'fb:publish-info')
            node = self._sub_element(parent, 'fb:city')
        if city:
            node.text = city
        else:
            node.getparent().remove(node)

    def set_publish_isbn(self, isbn):
        node = self._get('//fb:description/fb:publish-info/fb:isbn')
        if node is None:
            parent = self._get('//fb:description/fb:publish-info')
            if parent is None:
                descr_node = self._get('//fb:description')
                parent = self._sub_element(descr_node, 'fb:publish-info')
            node = self._sub_element(parent, 'fb:isbn')
        if isbn:
            node.text = isbn
        else:
           node.getparent().remove(node) 

    def set_publish_series(self, series):
        node = self._get('//fb:description/fb:publish-info/fb:sequence')
        if series:
            if node is None:
                parent = self._get('//fb:description/fb:publish-info')
                if parent is None:
                    descr_node = self._get('//fb:description')
                    parent = self._sub_element(descr_node, 'fb:publish-info')
                node = self._sub_element(parent, 'fb:sequence')
            node.attrib['name'] = series
        else:
            if node is not None:
                node.getparent().remove(node)

    def set_publish_series_index(self, series_index):
        node = self._get('//fb:description/fb:publish-info/fb:sequence')
        if series_index:
            if node is None:
                parent = self._get('//fb:description/fb:publish-info')
                if parent is None:
                    descr_node = self._get('//fb:description')
                    parent = self._sub_element(descr_node, 'fb:publish-info')
                node = self._sub_element(parent, 'fb:sequence')
            node.attrib['number'] = str(series_index)
        else:
            if node is not None:
                node.getparent().remove(node)

    ######## Service methods ########
    def save(self):
        if is_zipfile(self.file):
            zipfile = ZipFile(self.file, mode='w')
            zipfile.writestr(self.zip_file_info,
                       etree.tostring(self.tree, encoding=self.encoding,
                                      method='xml', xml_declaration=True, pretty_print=True))
            zipfile.close()
        else:
            self.tree.write(self.file, encoding=self.encoding, method='xml', 
                            xml_declaration=True, pretty_print=True)

    def _get_person(self, node):
        first_name = ''
        middle_name = ''
        last_name = ''

        for e in node:
            if etree.QName(e).localname == 'first-name':
                first_name = xstr(e.text)
            elif etree.QName(e).localname == 'middle-name':
                middle_name = xstr(e.text)
            elif etree.QName(e).localname == 'last-name':
                last_name = xstr(e.text)

        author = '{0} {1} {2}'.format(first_name, middle_name, last_name)
        return ' '.join(author.split())

    def _set_person(self, node, person):

        first_name = ''
        middle_name = ''
        last_name = ''

        person_parts = person.split()
        if len(person_parts) == 3:
            first_name = person_parts[0]
            middle_name = person_parts[1]
            last_name = person_parts[2]
        elif len(person_parts) == 2:
            first_name = person_parts[0]
            last_name = person_parts[1]
        else:
            last_name = person
    
        if first_name:
            self._sub_element(node, 'fb:first-name').text = first_name.strip()
        if middle_name:
            self._sub_element(node, 'fb:middle-name').text = middle_name.strip()
        if last_name:
            self._sub_element(node, 'fb:last-name').text = last_name.strip()


    def _get(self, xpath):
        node_list = self.tree.xpath(xpath, namespaces=ns_map)
        for node in node_list:
            return node

    def _get_all(self, xpath):
        return self.tree.xpath(xpath, namespaces=ns_map)

    def _sub_element(self, parent, name):
        ns, tag = name.split(':')
        return etree.SubElement(parent, etree.QName(ns_map[ns], tag))



from lxml import etree
import hashlib

from .fb2genres import fb2genres
from .exceptions import BadLanguage
from .utils import str_to_list, replace_keywords, split_ext, normalize_path

class PublishInfo:
    def __init__(self):
        self.title = None
        self.publisher = None
        self.year = None
        self.city = None
        self.series = None
        self.series_index = None
        self.isbn = None

    def __str__(self):
        result = []
        for key in self.__dict__.keys():
                result.append('{0}: {1}'.format(key, self.__dict__[key]))

        return '[' + ', '.join(result) + ']'

class Metadata:
    def __init__(self):
        self.identifier = None
        self.title = None
        self.author_list = []
        self.author_sort_list = []
        self.translator_list = []
        self.series = None
        self.series_index = None
        self.tag_list = []
        self.description = None
        self.lang = None
        self.format = None
        self.format_version = None
        self.cover_image_data = None
        self.cover_file_name = None
        self.cover_media_type = None
        self.file = None
        self.publish_info = PublishInfo()
        self.file_created = None
        self.file_modified = None

    def author_list_to_string(self):
        return ', '.join(self.author_list) if self.author_list else []

    def translator_list_to_string(self):
        return ', '.join(self.translator_list) if self.translator_list else []

    def tag_list_to_string(self):
        return ', '.join(self.tag_list) if self.tag_list else []

    def tag_description_list_to_string(self, lang='ru'):
        if lang not in ('ru', 'en'):
            raise BadLanguage('Only ru and en languages supports')
        result = []
        tree = etree.fromstring(fb2genres, parser=etree.XMLParser())
        xpath_str = '//fbgenrestransfer/genre/subgenres/subgenre[@value="{}"]/genre-descr[@lang="{}"]/@title'
        for tag in self.tag_list:
            node = tree.xpath(xpath_str.format(tag, lang))
            try:
                result.append(str(node[0]))
            except IndexError:
                result.append(tag)
        return ', '.join(result)

    def set_author_list_from_string(self, s):
        self.author_list = str_to_list(s)

    def set_translator_list_from_string(self, s):
        self.translator_list = str_to_list(s)

    def set_tag_list_from_string(self, s):
        self.tag_list = str_to_list(s)

    def get_filename_by_pattern(self, filename_pattern, author_pattern, padnum=2):
        d = { '#Title': '', '#Series': '', '#Abbrseries': '',
              '#Number': '', '#Padnumber': '',
              '#Author': '', '#Authors': '', '#Translator': '',
              '#Bookid': '', '#Md5': ''
            }

        d['#Title'] = self.title
        d['#Author'] = self._get_authors_by_pattern(author_pattern, short=True)
        d['#Authors'] = self._get_authors_by_pattern(author_pattern, short=False)
        d['#Bookid'] = self.identifier

        if self.series:
            d['#Series'] = self.series
            abbr = ''.join(w[0] for w in self.series.split())
            d['#Abbrseries'] = abbr

            if self.series_index:
                d['#Number'] = str(self.series_index)
                d['#Padnumber'] = str(self.series_index).strip().zfill(padnum)

            if len(self.translator_list) > 0:
                try:
                    d['#Translator'] = self.translator_list[0].split()[-1]
                except:
                    d['#Translator'] = ''

        with open(self.file, 'rb') as f:
            data = f.read()
            d['#Md5'] = hashlib.md5(data).hexdigest()

        cases_d = {}
        for key, value in d.items():
            cases_d[key.lower()] = value.lower()
            cases_d[key.upper()] = value.upper()
        d.update(cases_d)

        file_ext = split_ext(self.file)
        result = replace_keywords(filename_pattern, d).strip() + file_ext 
 
        return normalize_path(result)


    def _get_authors_by_pattern(self, pattern, short=True):
        if len(self.author_list) == 0:
            return ''

        if short and len(self.author_list) > 1:
            if self.lang == 'ru':
                return replace_keywords(pattern, self._get_person_dict(self.author_list[0])) + ' и др'
            else:
                return replace_keywords(pattern, self._get_person_dict(self.author_list[0])) + ', et al'
        else:
            result = []
            for author in self.author_list:
                result.append(replace_keywords(pattern, self._get_person_dict(author)))
            return ', '.join(result)


    def _get_person_dict(self, person):
        d = { '#f': '', '#m': '', '#l': '', '#fi': '', '#mi':'' }
 
        person_parts = person.split()
        if len(person_parts) == 3:
            d['#f'] = person_parts[0]
            d['#m'] = person_parts[1]
            d['#l'] = person_parts[2]
        elif len(person_parts) == 2:
            d['#f'] = person_parts[0]
            d['#l'] = person_parts[1]
        else:
            d['#l'] = person

        if len(d['#f']) > 0:
            d['#fi'] = d['#f'][0] + '.'
        if len(d['#m']) > 0: 
            d['#mi']= d['#m'][0] + '.'
        return d

    def __str__(self):
        result = []
        for key in self.__dict__.keys():
            if key == 'cover_image_data':
                if self.__dict__[key] is not None:
                    result.append('{0}: {1}'.format(key, '<binary_data>')) 
                else:
                    result.append('{0}: {1}'.format(key, 'None')) 
            else:
                result.append('{0}: {1}'.format(key, self.__dict__[key]))

        return '[' + ', '.join(result) + ']'

    
    




import os
import posixpath
import tempfile
import shutil
from lxml import etree, html
from .myzipfile import ZipFile, is_zipfile, ZIP_DEFLATED, ZIP_STORED
from .utils import xstr

ns_map = {
        'n': 'urn:oasis:names:tc:opendocument:xmlns:container',
        'opf': 'http://www.idpf.org/2007/opf',
        'dc': 'http://purl.org/dc/elements/1.1/'
    }

class Epub2():
    def __init__(self, file):
        self.file = file
        self.opf = None
        self.content_root = None
        self.tree = None
        self.version = None
        self.cover_href = None
        self.cover_data = None
        
        if not is_zipfile(self.file):
            raise Exception('"{}" is not epub file.'.format(self.file))
        
        content = self._get_file_content('META-INF/container.xml')
        tree = etree.fromstring(content)
        self.opf = tree.xpath('n:rootfiles/n:rootfile/@full-path', namespaces=ns_map)[0]
        self.content_root = os.path.dirname(self.opf) + '/'
        if self.content_root == '/':
            self.content_root = ''
        content = self._get_file_content(self.opf)
        self.tree = etree.fromstring(content)
        self.version = self.tree.xpath('/opf:package/@version', namespaces=ns_map)[0]

    ########## Getters ##########
    def get_title(self):
        node = self._get('opf:metadata/dc:title')
        if node is not None:
            return xstr(node.text)
    
    def get_author_list(self):
        result = []
        node_list = self._get_all('opf:metadata/dc:creator[@opf:role="aut" or not(@opf:role)]')
        for node in node_list: result.append(xstr(node.text))
        return result
   
    def get_series(self):
        node = self._get('opf:metadata/opf:meta[@name="calibre:series"]')
        if node is not None:
            if 'content' in node.attrib:
                return xstr(node.attrib['content'])

    def get_series_index(self):
        node = self._get('opf:metadata/opf:meta[@name="calibre:series_index"]')
        if node is not None:
            if 'content' in node.attrib:
                return xstr(node.attrib['content'])

    def get_lang(self):
        node = self._get('opf:metadata/dc:language')
        if node is not None:
            return xstr(node.text)
    
    def get_tag_list(self):
        result = []
        node_list = self._get_all('opf:metadata/dc:subject')
        for node in node_list: result.append(xstr(node.text))
        return result

    def get_description(self):
        return xstr(self._get('opf:metadata/dc:description/text()'))

    def get_translator_list(self):
        result = []
        node_list = self._get_all('opf:metadata/dc:creator[@opf:role="trl"]')
        for node in node_list: result.append(xstr(node.text))
        return result

    def get_format(self):
        return 'epub'
    
    def get_format_version(self):
        return self.version

    def get_identifier(self):
         return  xstr(self._get('opf:metadata/dc:identifier/text()'))
         
    def get_cover_data(self):
        cover_id = None
        media_type = None
        href = None
        data = None

        node = self._get('opf:metadata/opf:meta[@name="cover"]')
        if node is not None:
            if 'content' in node.attrib: cover_id = node.attrib['content']
        if cover_id:
            node = self._get('opf:manifest/opf:item[@id="{0}"]'.format(cover_id))
            if node is not None:
                if 'href' in node.attrib: href = node.attrib['href']
                if 'media-type' in node.attrib: media_type = node.attrib['media-type']
            if href: data = self._get_file_content(self.content_root + href)
        else:
            (href, media_type, data) = self._get_cover_from_first_element()
        return (href, media_type, data)

    def _get_cover_from_first_element(self):
        media_type = None
        href = None
        img_href = None
        data = None

        nodes = self._get_all('opf:manifest/opf:item')
        for node in nodes:
            if node is not None:
                if 'media-type' in node.attrib: 
                    if node.attrib['media-type'] == 'application/xhtml+xml':
                        media_type = node.attrib['media-type']
                        break

        if media_type:
            href = node.attrib['href'] 
            if href:
                content = self._get_file_content(self.content_root + href)
                tree = html.fromstring(content)
                nodes = tree.xpath('//img')
                for node in nodes:
                    if 'src' in node.attrib:
                        img_href = node.attrib['src']
                        if img_href.lower().endswith(('.jpg', '.jpeg')): media_type = 'image/jpeg'
                        elif img_href.lower().endswith(('.png')): media_type = 'image/png'
                        else: media_type = None
                        break
                if not img_href:
                    nodes = tree.xpath('//image')
                    for node in nodes:
                        if 'xlink:href' in node.attrib:
                            img_href = node.attrib['xlink:href']
                            if img_href.lower().endswith(('.jpg', '.jpeg')): media_type = 'image/jpeg'
                            elif img_href.lower().endswith(('.png')): media_type = 'image/png'
                            else: media_type = None
                            break

                if media_type and img_href:
                    base_path = posixpath.join(self.content_root, os.path.dirname(href))
                    img_path = posixpath.normpath(posixpath.join(base_path, img_href))
                    data = self._get_file_content(img_path)
                    return (img_path, media_type, data)
        return (None, None, None)

    ########## Setters ##########
    def set_title(self, title):
        node = self._get('opf:metadata/dc:title')
        if node is None:
            meta_node = self._get('opf:metadata')
            node = self._sub_element(meta_node, 'dc:title')
        node.text = title

    def set_author_list(self, author_list):
        node_list = self._get_all('opf:metadata/dc:creator[@opf:role="aut" or not(@opf:role)]')
        for node in node_list: node.getparent().remove(node)
        meta_node = self._get('opf:metadata')
        for author in author_list:
            if author:
                node = self._sub_element(meta_node, 'dc:creator')
                node.attrib['role'] = 'aut'
                node.text = author

    def set_series(self, series):
        node = self._get('opf:metadata/opf:meta[@name="calibre:series"]')
        if series:
            if node is None:
                meta_node = self._get('opf:metadata')
                node = self._sub_element(meta_node, 'opf:meta')
                node.attrib['name'] = 'calibre:series'
            node.attrib['content'] = series
        else:
            if node is not None:
                node.getparent().remove(node)
    
    def set_series_index(self, series_index):
        node = self._get('opf:metadata/opf:meta[@name="calibre:series_index"]')
        if series_index:
            if node is None:
                meta_node = self._get('opf:metadata')
                node = self._sub_element(meta_node, 'opf:meta')
                node.attrib['name'] = 'calibre:series_index'
            node.attrib['content'] = str(series_index)
        else:
            if node is not None:
                node.getparent().remove(node)

    def set_lang(self, lang):
        node = self._get('opf:metadata/dc:language')
        if node is None:
            meta_node = self._get('opf:metadata')
            node = self._sub_element(meta_node, 'dc:language')
        node.text = lang

    def set_tag_list(self, tag_list):
        node_list = self._get_all('opf:metadata/dc:subject')
        for node in node_list: node.getparent().remove(node)
        meta_node = self._get('opf:metadata')
        for tag in tag_list:
            if tag:
                node = self._sub_element(meta_node, 'dc:subject')
                node.text = tag
                node.tail = '\n'

    def set_translator_list(self, translator_list):
        node_list = self._get_all('opf:metadata/dc:creator[@opf:role="trl"]')
        for node in node_list: node.getparent().remove(node)
        meta_node = self._get('opf:metadata')
        for translator in translator_list:
            if translator:
                node = self._sub_element(meta_node, 'dc:creator')
                node.attrib['role'] = 'trl'
                node.text = translator

    def set_cover_data(self, href, media_type, data):
        (href, _, _) = self.get_cover_data()

        if href and data:
            self.cover_href = href
            self.cover_data = data

    ########## Service methods ##########
    def save(self):
        temp_dir = tempfile.mkdtemp(prefix='em')
        dest_file = os.path.join(temp_dir, os.path.basename(self.file))
        src_zip = ZipFile(self.file, mode='r')
        dest_zip = ZipFile(dest_file, mode='w')
        try:
            for f in src_zip.infolist():
                if f.filename == self.opf:
                    dest_zip.writestr(self.opf, etree.tostring(self.tree, encoding='utf-8',
                                      method='xml', xml_declaration=True, pretty_print=True))

                elif f.filename == self.content_root + xstr(self.cover_href):
                    if self.cover_data:
                        dest_zip.writestr(self.content_root + self.cover_href, self.cover_data)
                elif f.filename == 'mimetype':
                    buf = src_zip.read(f)
                    dest_zip.writestr(f.filename, buf, ZIP_STORED)
                else:
                    buf = src_zip.read(f)
                    dest_zip.writestr(f.filename, buf, ZIP_DEFLATED)
            src_zip.close()
            dest_zip.close()
            shutil.copyfile(dest_file, self.file)
            shutil.rmtree(temp_dir)
        except Exception as e:
            src_zip.close()
            dest_zip.close()
            shutil.rmtree(temp_dir)
            raise Exception(repr(e))

    def _get_file_content(self, filename):
        zipfile = ZipFile(self.file)
        content = zipfile.read(filename)
        zipfile.close()
        return content

    def _get_all(self, xpath_str):
        return self.tree.xpath(xpath_str, namespaces=ns_map)

    def _get(self, xpath_str):
        node_list = self.tree.xpath(xpath_str, namespaces=ns_map)
        for node in node_list:
            return node

    def _sub_element(self, parent, name):
        ns, tag = name.split(':')
        return etree.SubElement(parent, etree.QName(ns_map[ns], tag))
